/**
  Example program to demonstrate simple kernel threads.
  Orion Sky Lawlor, olawlor@acm.org, 2006/02/03 (Public Domain)
*/
#include <stdio.h>
#include "osl/porthread.h"
#include "osl/porthread.cpp"

/* Tiny x86-specific lock library */
volatile int lock; /* ==0 -> unlocked.  ==1 -> locked. */

void mylock_lock(void) {
	while (lock==1) { porthread_yield(10); }
	/* DANGER! lock might get grabbed right here! */
	lock=1;
}
void mylock_unlock(void) {
	lock=0;
	porthread_yield(10); /* give others a chance to run */
}


volatile int iter;
enum {max_iter=10};
void thread_routineA(void *arg) {
	printf("Inside threadA\n"); fflush(stdout);
	for (iter=0;iter<max_iter;iter++) {
		printf("A wants lock (%d)\n",iter); fflush(stdout);
		mylock_lock();
		for (int j=0;j<10;j++) {
			printf("      A got lock (%d) %d\n",iter,j); fflush(stdout);
			porthread_yield(10);
		}
		mylock_unlock();
	}
	printf("A done\n");
}

void thread_routineB(void *arg) {
	printf("Inside threadB\n"); fflush(stdout);
	while (iter<max_iter) {
		printf("B wants lock (%d)\n",iter); fflush(stdout);
		mylock_lock();
		for (int j=0;j<10;j++) {
			printf("    B got lock (%d) %d\n",iter,j); fflush(stdout);
		}
		mylock_unlock();
	}
	printf("B done\n");
}


int main() {
	porthread_t a=porthread_create(thread_routineA,(void *)0);
	porthread_t b=porthread_create(thread_routineB,(void *)1);
	printf("Done creating threads\n");
	porthread_wait(a);
	porthread_wait(b);
	return 0;
}
